(***********************************************************************)
(*                                                                     *)
(*                           CIME Caml                                 *)
(*                                                                     *)
(*            Pierre Weis, projet Cristal, INRIA Rocquencourt          *)
(*                                                                     *)
(*  Copyright 2009 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  Distributed only by permission.                   *)
(*                                                                     *)
(***********************************************************************)

(* $Id: preprocessor.mli,v 1.6 2015/11/18 19:17:22 weis Exp $ *)

(** {6 Pre-processing modes for Htmlc} *)

(** The pre-processing modes for Htmlc are the following:
  - [Mode_verbatim] means no pre-processing.
  - [Mode_source] means pre-processing using the list of pre-processors
    specified for source files.
  - [Mode_ssi] means pre-processing using the list of pre-processors
    specified for file inclusion from a SSI directive.
  - [Mode_non_ssi] means pre-processing using the list of pre-processors
    specified for file inclusion from a non-SSI directive. *)

(** {6 Gathering pre-processor commands} *)

(** Adding pre-processor commands to the various lists of pre-processors
  that should be applied to file arguments. *)

val add_source_preprocessor_command : Htmlc_types.command_name -> unit
(** Adds a pre-processor for [Mode_source] pre-processing mode. *)
;;

val add_ssi_preprocessor_command : Htmlc_types.command_name -> unit
(** Adds a pre-processor for [Mode_ssi] pre-processing mode. *)
;;

val add_non_ssi_preprocessor_command : Htmlc_types.command_name -> unit
(** Adds a pre-processor for [Mode_non_ssi] pre-processing mode. *)
;;

(** {6 Applying pre-processors} *)

val fold :
  Htmlc_types.command_name list -> File.file_name ->
  Scanf.Scanning.scanbuf -> Scanf.Scanning.scanbuf
(** [fold prepros fname sb] calls in turn each command in the
  list of pre-processors [prepros] on the given scan buffer [sb].
  The result of each pre-processor call is collected in an output buffer,
  then extracted from the buffer and turned into a new scan buffer to call
  the remaining list of pre-processors.

  The result is a new scan buffer containing the pre-processed file.

  Note: presumably, the [sb] argument is reading from file [fname], but
  [Preprocesor.fold] does not requires it. *)
;;

val call_preprocessors :
  Htmlc_types.htmlc_mode ->
  File.file_name ->
  Scanf.Scanning.scanbuf -> Scanf.Scanning.scanbuf
(** [call_preprocessors hmode src_name sb] calls in turn each command in the
  list of pre-processors associated to pre-processing mode [hmode] on the
  given scan buffer [sb].

  The result is a new scan buffer containing the pre-processed file.

  Note: presumably, the [sb] argument is reading from file [fname], but
  [Preprocesor.fold] does not requires it. *)
;;

val call_source_preprocessors :
  File.file_name ->
  Scanf.Scanning.scanbuf -> Scanf.Scanning.scanbuf
(** Pre-processing files with the pre-processor commands for [Mode_source]
  pre-processing mode.

  Note: presumably, the [sb] argument is reading from file [fname], but
  [Preprocesor.fold] does not requires it. *)
;;

val call_ssi_preprocessors :
  File.file_name ->
  Scanf.Scanning.scanbuf -> Scanf.Scanning.scanbuf
(** Pre-processing files with the pre-processor commands for [Mode_ssi]
  pre-processing mode.

  Note: presumably, the [sb] argument is reading from file [fname], but
  [Preprocesor.fold] does not requires it. *)
;;

val call_non_ssi_preprocessors :
  File.file_name ->
  Scanf.Scanning.scanbuf -> Scanf.Scanning.scanbuf
(** Pre-processing files with the pre-processor commands for [Mode_non_ssi]
  pre-processing mode.

  Note: presumably, the [sb] argument is reading from file [fname], but
  [Preprocesor.fold] does not requires it. *)
;;
