(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uarray.ml,v 1.50 2017/10/31 12:42:59 fclement Exp $ *)

(* More array operations. *)

module A = Array;;

module Nat = Unatural;;
module L = Ulist;;

let _0_ = Nat._0_
and _1_ = Nat._1_
and _2_ = Nat._2_
;;

let nat_pred2 i = Nat.sub i _2_;;

let apply f x = f x;;

let qual = Upervasives.qualify "Uarray";;
let invalid_arg = Upervasives.invalid_arg_qual qual
and failwith = Upervasives.failwith_qual qual
;;

type 'a t = 'a array;;

type length = Upervasives.length
and index = Upervasives.index
and offset = Upervasives.offset
;;

external lti : length -> int = "%identity";;
external iti : index -> int = "%identity";;
external oti : offset -> int = "%identity";;
external uloi : int -> length = "%identity";;
external uioi : int -> index = "%identity";;

let length a = uloi (A.length a)
and get a n = A.get a (iti n)
and set a n x = A.set a (iti n) x
and unsafe_get a n = A.unsafe_get a (iti n)
and unsafe_set a n x = A.unsafe_set a (iti n) x
and make n = A.make (iti n)
and init n f = A.init (iti n) (fun i -> f (uioi i))
and append = A.append
and concat = A.concat
and copy = A.copy
and blit a1 ofs1 a2 ofs2 len = A.blit a1 (oti ofs1) a2 (oti ofs2) (lti len)
and of_list = A.of_list
and to_list = A.to_list
and iter = A.iter
and iter_i f = A.iteri (fun i -> f (uioi i))
and map = A.map
and map_i f = A.mapi (fun i -> f (uioi i))
and fold_left = A.fold_left
and fold_right = A.fold_right
and sort = A.sort
and stable_sort = A.stable_sort
and fast_sort = A.fast_sort
;;

let empty = [||];;

let init_rec n f =
  if n = _0_ then [||] else
  (* Now, we have n > 0.
     Therefore, the following Nat.pred will not fail. *)
  let res = make  n (f [||] _0_) in
  Nat.for_loop _1_ (Nat.pred n)
    (fun i ->
      (* Now, we have 1 <= i < n.
         Therefore, the following unsafe_set will be safe. *)
      unsafe_set res i (f res i));
  res
;;

type 'a init_rec1 = 'a
and 'a prev_rec1 = 'a
;;

let init_rec1 n a0 f =
  let ff a i =
    if i = _0_ then a0 else
    (* Now, we have 0 < i < n.
       Therefore, the following Nat.pred will not fail, and
       the following unsafe_get will be safe. *)
    let aim1 = unsafe_get a (Nat.pred i) in
    f aim1 i in
  init_rec n ff
;;

type 'a scheme_rec1_lin = 'a;;
type 'a prod_scal1 = 'a scheme_rec1_lin -> 'a prev_rec1 -> 'a;;

let init_rec1_lin n init scheme scal =
  let f prev _i = scal scheme prev in
  init_rec1 n init f
;;

type 'a init_rec2 = 'a * 'a
and 'a prev_rec2 = 'a * 'a
;;

let init_rec2 n (a0, a1) f =
  let ff a i =
    if i = _0_ then a0 else
    if i = _1_ then a1 else
    (* Now, we have 2 <= i < n.
       Therefore, the following nat_pred2 and Nat.pred will not fail, and
       the following unsafe_get will be safe. *)
    let aim2 = unsafe_get a (nat_pred2 i) in
    let aim1 = unsafe_get a (Nat.pred i) in
    f (aim2, aim1) i in
  init_rec n ff
;;

type 'a scheme_rec2_lin = 'a * 'a;;
type 'a prod_scal2 = 'a scheme_rec2_lin -> 'a prev_rec2 -> 'a;;

let init_rec2_lin n init scheme scal =
  let f prev _i = scal scheme prev in
  init_rec2 n init f
;;

let is_empty a = length a = _0_;;

let check_nonempty fun_name a =
  if is_empty a then Upervasives.invalid_arg fun_name "empty array"
;;

let check_subarray fun_name a ofs len =
  if Nat.add ofs len > length a then
    invalid_arg fun_name "wrong subarray specification"
;;

let sub a ofs len =
  check_subarray "sub" a ofs len;
  (* Now, we have ofs + len <= length a.
     Thus, for all 0 <= i < len, we have
     0 <= ofs <= ofs + i < ofs + len <= length a.
     Therefore, the following unsafe_get will be safe. *)
  init len (fun i -> unsafe_get a (Nat.add ofs i))
;;

type index_min = index
and index_max = index
;;

let nat_for_loop = Nat.for_loop
and nat_rev_for_loop = Nat.rev_for_loop
;;

let for_loop ofs len body =
  if len = _0_ then () else
  (* Now, we have len > 0.
     Therefore, the following Nat.pred will not fail. *)
  nat_for_loop ofs (Nat.add ofs (Nat.pred len)) body
and rev_for_loop ofs len body =
  if len = _0_ then () else
  (* Now, we have len > 0.
     Therefore, the following Nat.pred will not fail. *)
  nat_rev_for_loop (Nat.add ofs (Nat.pred len)) ofs body
;;

let get_same qual fun_name gen_msg f a =
  let len = length a in
  if len = _0_ then
    Upervasives.invalid_arg_qual qual fun_name "empty array" else
  (* Now, we have len > 0, .
     Therefore, the following unsafe_get will be safe and Nat.pred will not
     fail. *)
  let fa0 = f (unsafe_get a _0_) in
  for_loop _1_ (Nat.pred len)
    (fun i ->
      (* Now, we have 1 <= i <= len - 1 < len = length a.
         Therefore, the following unsafe_get will be safe. *)
      let fai = f (unsafe_get a i) in
      if fai <> fa0 then
        Upervasives.invalid_arg_qual qual fun_name
          (gen_msg i fai fa0));
  fa0
;;

(* Filling arrays. *)

let unsafe_sub_fill_i f a ofs len =
  for_loop ofs len (fun i -> set a i (f i (get a i)))
;;

let sub_fill_i f a ofs len =
  check_subarray "sub_fill_i" a ofs len;
  unsafe_sub_fill_i f a ofs len
;;

let fill_i f a = unsafe_sub_fill_i f a _0_ (length a);;

let unsafe_sub_fill f = unsafe_sub_fill_i (fun _ ai -> f ai);;

let sub_fill f a ofs len =
  check_subarray "sub_fill" a ofs len;
  unsafe_sub_fill f a ofs len
;;

let fill f a = unsafe_sub_fill f a _0_ (length a);;

(* Converters. *)

let list_of_list_array la = to_list (map of_list la);;

let list_to_list_array al = of_list (L.map to_list al);;

let of_list1 (x, l as l1) =
  let a1 = make (Ulist1.length l1) x in
  (* In the following, we have 0 <= i < length l = length l1 - 1.
     Thus, we have 1 <= i + 1 < length l1 = length a1.
     Therefore, the following unsafe_set will be safe. *)
  L.iter_i (fun i x -> unsafe_set a1 (Nat.succ i) x) l;
  a1
;;

(* Iterators. *)

let gen_sub_iter_i fun_name f a ofs len =
  check_subarray fun_name a ofs len;
  (* Now, we have ofs + len <= length a.
     Thus, for all ofs <= i <= ofs + len - 1, we have
     0 <= ofs <= i < ofs + len <= length a.
     Therefore, the following unsafe_get will be safe. *)
  for_loop ofs len (fun i -> f i (unsafe_get a i))
;;

let sub_iter_i f = gen_sub_iter_i "sub_iter_i" f
and sub_iter f = gen_sub_iter_i "sub_iter" (fun _ x -> f x)
;;

let gen_sub_fold_left_i fun_name f accu a ofs len =
  check_subarray fun_name a ofs len;
  (* Now, we have ofs + len <= length a.
     Thus, for all ofs <= i <= ofs + len - 1, we have
     0 <= ofs <= i < ofs + len <= length a.
     Therefore, the following unsafe_get will be safe. *)
  let r = ref accu in
  for_loop ofs len (fun i -> r := f !r i (unsafe_get a i));
  !r
;;

let sub_fold_left_i f =
  gen_sub_fold_left_i "sub_fold_left_i" f

and sub_fold_left f =
  gen_sub_fold_left_i "sub_fold_left" (fun accu _ x -> f accu x)

and fold_left_i f accu a =
  gen_sub_fold_left_i "fold_left_i" f accu a _0_ (length a)
;;

let gen_sub_fold_right_i fun_name f a ofs len accu =
  check_subarray fun_name a ofs len;
  (* Now, we have ofs + len <= length a.
     Thus, for all ofs <= i <= ofs + len - 1, we have
     0 <= ofs <= i < ofs + len <= length a.
     Therefore, the following unsafe_get will be safe. *)
  let r = ref accu in
  for_loop ofs len (fun i -> r := f i (unsafe_get a i) !r);
  !r
;;

let sub_fold_right_i f =
  gen_sub_fold_right_i "sub_fold_right_i" f

and sub_fold_right f =
  gen_sub_fold_right_i "sub_fold_right" (fun _ x -> f x)

and fold_right_i f a =
  gen_sub_fold_right_i "fold_right_i" f a _0_ (length a)
;;

(* Iterators on two arrays. *)

let check_same_length fun_name a1 a2 =
  let len = length a1 in
  if length a2 <> len then
    invalid_arg fun_name "array arguments have different length" else
  len
;;

let gen_sub_iter2_i fun_name f a1 ofs1 a2 ofs2 len =
  check_subarray fun_name a1 ofs1 len;
  (* Now, we have ofs1 + len <= length a1.
     Thus, for all 0 <= i < len, we have
     0 <= ofs1 <= i1 = ofs1 + i < ofs1 + len <= length a1.
     Therefore, the following (unsafe_get a1) will be safe. *)
  check_subarray fun_name a2 ofs2 len;
  (* Now, we have ofs2 + len <= length a2.
     Thus, for all 0 <= i < len, we have
     0 <= ofs2 <= i2 = ofs2 + i < ofs2 + len <= length a2.
     Therefore, the following (unsafe_get a2) will be safe. *)
  for_loop _0_ len
    (fun i ->
      let i1 = Nat.add ofs1 i
      and i2 = Nat.add ofs2 i in
      f i1 (unsafe_get a1 i1) i2 (unsafe_get a2 i2))
;;

let sub_iter2_i f = gen_sub_iter2_i "sub_iter2_i" f
and sub_iter2 f = gen_sub_iter2_i "sub_iter2" (fun _ x1 _ x2 -> f x1 x2)
;;

let gen_iter2_i fun_name f a1 a2 =
  let len = check_same_length fun_name a1 a2 in
  sub_iter2_i (fun i x1 _ x2 -> f i x1 x2) a1 _0_ a2 _0_ len
;;

let iter2_i f = gen_iter2_i "iter2_i" f
and gen_iter2 fun_name f = gen_iter2_i fun_name (fun _ x1 x2 -> f x1 x2)
;;

let iter2 f = gen_iter2 "iter2" f
and iter_rails fa = gen_iter2 "iter_rails" apply fa
;;

let gen_map2_i fun_name f a1 a2 =
  let len = check_same_length fun_name a1 a2 in
  (* Now, we have len = length a1 = length a2.
     Thus, in the following, we have 0 <= i < len = length a1 = length a2.
     Therefore, both unsafe_get will be safe. *)
  init len (fun i -> f i (unsafe_get a1 i) (unsafe_get a2 i))
;;

let map2_i f = gen_map2_i "map2_i" f
and gen_map2 fun_name f = gen_map2_i fun_name (fun _ x1 x2 -> f x1 x2)
;;

let map2 f = gen_map2 "map2" f
and rails fa = gen_map2 "rails" apply fa
;;

let fold_left2 f accu a1 a2 =
  let len = check_same_length "fold_left2" a1 a2 in
  (* Now, we have len = length a1 = length a2.
     Thus, in the following, we have 0 <= i < len = length a1 = length a2.
     Therefore, both unsafe_get will be safe. *)
  let r = ref accu in
  for_loop _0_ len
    (fun i -> r := f !r (unsafe_get a1 i) (unsafe_get a2 i));
  !r
;;

let fold_right2 f a1 a2 accu =
  let len = check_same_length "fold_right2" a1 a2 in
  (* Now, we have len = length a1 = length a2.
     Thus, in the following, we have 0 <= i < len = length a1 = length a2.
     Therefore, both unsafe_get will be safe. *)
  let r = ref accu in
  rev_for_loop _0_ len
    (fun i -> r := f (unsafe_get a1 i) (unsafe_get a2 i) !r);
  !r
;;

(* Iterators on more arrays. *)

let iter3 f a1 a2 a3 =
  let _len = check_same_length "iter3" a1 a2 in
  let len = check_same_length "iter3" a2 a3 in
  (* Now, we have len = length a1 = length a2 = length a3.
     Thus, in the following, we have
     0 <= i < len = length a1 = length a2 = length a3.
     Therefore, all three unsafe_get will be safe. *)
  for_loop _0_ len
    (fun i -> f (unsafe_get a1 i) (unsafe_get a2 i) (unsafe_get a3 i))
;;

let iter4 f a1 a2 a3 a4 =
  let _len = check_same_length "iter4" a1 a2 in
  let _len = check_same_length "iter4" a2 a3 in
  let len = check_same_length "iter4" a3 a4 in
  (* Now, we have len = length a1 = length a2 = length a3 = length a4.
     Thus, in the following, we have
     0 <= i < len = length a1 = length a2 = length a3 = length a4.
     Therefore, all four unsafe_get will be safe. *)
  for_loop _0_ len
    (fun i ->
      f (unsafe_get a1 i) (unsafe_get a2 i)
        (unsafe_get a3 i) (unsafe_get a4 i))
;;

let map3 f a1 a2 a3 =
  let _len = check_same_length "map3" a1 a2 in
  let len = check_same_length "map3" a2 a3 in
  (* Now, we have len = length a1 = length a2 = length a3.
     Thus, in the following, we have
     0 <= i < len = length a1 = length a2 = length a3.
     Therefore, all three unsafe_get will be safe. *)
  init len
    (fun i -> f (unsafe_get a1 i) (unsafe_get a2 i) (unsafe_get a3 i))
;;

let map4 f a1 a2 a3 a4 =
  let _len = check_same_length "map4" a1 a2 in
  let _len = check_same_length "map4" a2 a3 in
  let len = check_same_length "map4" a3 a4 in
  (* Now, we have len = length a1 = length a2 = length a3 = length a4.
     Thus, in the following, we have
     0 <= i < len = length a1 = length a2 = length a3 = length a4.
     Therefore, all four unsafe_get will be safe. *)
  init len
    (fun i ->
       f (unsafe_get a1 i) (unsafe_get a2 i)
         (unsafe_get a3 i) (unsafe_get a4 i))
;;

(* Iterators on list of arrays. *)

let map_list f al = A.map f (list_to_list_array al);;

(* Array scanning. *)

let for_all p a =
  let len = length a in
  let rec loop i =
    (* i is increasing from 0. *)
    i >= len ||
    (* Now, we have 0 <= i < len = length a.
       Therefore, the following unsafe_get will be safe. *)
    p (unsafe_get a i) && loop (Nat.succ i) in
  loop _0_
;;

let exists p a =
  let len = length a in
  let rec loop i =
    (* i is increasing from 0. *)
    i < len &&
    (* Now, we have 0 <= i < len = length a.
       Therefore, the following unsafe_get will be safe. *)
    (p (unsafe_get a i) || loop (Nat.succ i)) in
  loop _0_
;;

let mem x = exists (( = ) x)
and memq x = exists (( == ) x)
;;

let unsafe_sub_find p a ofs len =
  (* unsafe_sub_find is always called with
     0 <= ofs <= ofs + len <= length a. *)
  let lim = Nat.add ofs len in
  let rec loop i =
    (* i is increasing from ofs. *)
    if i >= lim then raise Not_found else
    (* Now, we have 0 <= ofs <= i < lim = ofs + len <= length a.
       Therefore, the following unsafe_get will be safe. *)
    let ai = unsafe_get a i in
    if p ai then ai else loop (Nat.succ i) in
  loop ofs
;;

let sub_find p a ofs len =
  check_subarray "sub_find" a ofs len;
  (* Now, we have ofs + len <= length a.
     Therefore, unsafe_get in unsafe_sub_find will be safe. *)
  unsafe_sub_find p a ofs len
;;

(* In the following, we have 0 + length a = length a <= length a.
   Therefore, unsafe_get in unsafe_sub_find will be safe. *)
let find p a = unsafe_sub_find p a _0_ (length a);;

let unsafe_rev_sub_find_all_i p a ofs len =
  (* unsafe_rev_sub_find_all is always called with
     0 <= ofs <= ofs + len <= length a. *)
  let lim = Nat.add ofs len in
  let rec loop accu i =
    (* i is increasing from ofs. *)
    if i >= lim then accu else
    (* Now, we have 0 <= ofs <= i < lim = ofs + len <= length a.
       Therefore, the following unsafe_get will be safe. *)
    let ai = unsafe_get a i in
    let accu = if p i ai then ai :: accu else accu in
    loop accu (Nat.succ i) in
  loop [] ofs
;;

let unsafe_rev_sub_find_all p = unsafe_rev_sub_find_all_i (fun _ ai -> p ai);;

let rev_sub_find_all_i p a ofs len =
  check_subarray "rev_sub_find_all_i" a ofs len;
  (* Now, we have ofs + len <= length a.
     Therefore, unsafe_get in unsafe_rev_sub_find will be safe. *)
  unsafe_rev_sub_find_all_i p a ofs len
;;

let rev_sub_find_all p a ofs len =
  check_subarray "rev_sub_find_all" a ofs len;
  (* Now, we have ofs + len <= length a.
     Therefore, unsafe_get in unsafe_rev_sub_find will be safe. *)
  unsafe_rev_sub_find_all p a ofs len
;;

(* In the following, we have 0 + length a = length a <= length a.
   Therefore, unsafe_get in unsafe_rev_sub_find will be safe. *)
let rev_find_all_i p a = unsafe_rev_sub_find_all_i p a _0_ (length a);;

(* In the following, we have 0 + length a = length a <= length a.
   Therefore, unsafe_get in unsafe_rev_sub_find will be safe. *)
let rev_find_all p a = unsafe_rev_sub_find_all p a _0_ (length a);;

let for_all2 p a1 a2 =
  let len = check_same_length "for_all2" a1 a2 in
  (* Now, we have len = length a1 = length a2. *)
  let rec loop i =
    (* i is increasing from 0. *)
    i >= len ||
    (* Now, we have 0 <= i < len = length a1 = length a2.
       Therefore, both unsafe_get will be safe. *)
    p (unsafe_get a1 i) (unsafe_get a2 i) && loop (Nat.succ i) in
  loop _0_
;;

(* Selecting elements in arrays. *)

let gen_select fname f chi a =
  ignore (check_same_length fname chi a);
  let ml =
    rev_find_all_i (fun i _ -> unsafe_get chi i = Upervasives.Keep) a in
  of_list (Ulist.rev_map f ml)
;;

let map_select f = gen_select "map_select" f
and select chi = gen_select "select" Upervasives.id chi
;;

let unselect x chi a =
  let fw = failwith "unselect" in
  let len_chi = length chi in
  let len_a = length a in
  let res = make len_chi x in
  let nb_keep = ref _0_ in
  for_loop _0_ len_chi
    (fun i ->
      (* Now, we have 0 <= i < len_chi = length res.
         Therefore, the following ``unsafe_get chi'' and ``unsafe_set res''
         will not fail. *)
      if !nb_keep >= len_a then
        fw "the number of Keep values in 2nd argument is greater than \
           the length of 3rd argument" else
      (* Now, we have 0 <= !nb_keep < len_a.
         Therefore, the following ``unsafe_get a'' will not fail. *)
      match unsafe_get chi i with
      | Upervasives.Keep ->
          unsafe_set res i (unsafe_get a !nb_keep);
          Nat.incr nb_keep
      | Upervasives.Forget -> ());
  if !nb_keep = len_a then res else
  fw "the number of Keep values in 2nd argument is less than \
     the length of the 3rd argument"
;;

(* Sorting arrays. *)

let is_sorted cmp a =
  let len = length a in
  if len = _0_ then true else
  (* Now, we have len > 0.
     Therefore, the following unsafe_get will be safe and the folowing
     Nat.pred will not fail. *)
  let a0 = unsafe_get a _0_ in
  let (sorted, _) =
    sub_fold_left
      (fun (b, aim1) ai -> (b && cmp aim1 ai, ai))
      (true, a0) a _1_ (Nat.pred len) in
  sorted
;;

let is_strictly_increasing a = is_sorted (fun x y -> x < y) a;;

let check_strictly_increasing fun_name a =
  if not (is_strictly_increasing a) then
    invalid_arg fun_name "elements of array are not strictly increasing"
;;

let gen_sorted_array_find eq a x =
  let rec dicho l r =
    (* l is increasing from 0, and r is decreasing from length a. *)
    if l >= r then raise Not_found else
    (* Now, we have 0 <= l < r <= length a. *)
    let ip = Nat.div (Nat.add l r) _2_ in
    (* Now, we have 0 <= l <= ip < r <= length a.
       Therefore, the following unsafe_get will be safe. *)
    let p = unsafe_get a ip in
    if eq x p then ip else
    if x < p then dicho l ip else
    dicho (Nat.succ ip) r in
  dicho _0_ (length a)
;;

let sorted_array_find a = gen_sorted_array_find ( = ) a
and sorted_array_findq a = gen_sorted_array_find ( == ) a
;;
