(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: udebug.mli,v 1.55 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 Report/debug/warning/error option flags and printing facilities}

 This module provides functionalities related to the set up of specific
 warn error, reporting, and debugging option flags (either marked for
 {i application} or {i library} purpose, see module {!Ucommand_option} from
 the OCaml user's additional library Ulib) to be used on the command line,
 and specific printing facilities for reporting/monitoring, debugging, and
 printing warning and error messages.
 Messages are printed on stderr.

 This module is based on modules {!Upervasives}, {!Uaref}, {!Uoption_key}, and
 {!Ucommand_option} (from Ulib). *)

(** {6 Messages with no tags} *)

type 'a print = 'a -> unit
and 'a gen_print = Upervasives.function_name option -> 'a print
;;

type message_print = Upervasives.message print
and gen_message_print = Upervasives.gen_message print
and message_gen_print = Upervasives.message gen_print
and gen_message_gen_print = Upervasives.gen_message gen_print
;;

type printing_facilities = {
  get_warn_error : unit -> bool;
  report : message_print;
  debug : message_gen_print;
  warning : message_gen_print;
  error : message_gen_print;
  report_action : gen_message_print;
  debug_action : gen_message_gen_print;
  warning_action : gen_message_gen_print;
  error_action : gen_message_gen_print;
}
(** The type of printing facilities.
 Prefixes following the specification described above are automatically added.
 Field [get_warn_error] returns the warn error mode.
 Field [report] is a printing function taking the message to be prefixed as
 argument.
 Fields [debug], [warning], and [error] are similar to [report] but they take
 the function name as an additional first argument.
 Fields [report_action], [debug_action], [warning_action], and [error_action]
 are similar to [report], [debug], [warning], and [error] with their message
 argument replaced by a message generator argument. *)
;;

val application_printing_facilities :
  Ucommand_option.software_option_key_data -> printing_facilities
;;
val library_printing_facilities :
  Ucommand_option.software_option_key_data -> printing_facilities
(** Printing facilities construction functions with no tag prefix.
 Warn error, reporting, and debugging option flags to be used on the command
 line are automatically computed from the argument. *)
;;

(** {6 Messages with static tags} *)

val application_printing_facilities_with_static_tag :
  Ucommand_option.software_option_key_data -> printing_facilities
;;
val library_printing_facilities_with_static_tag :
  Ucommand_option.software_option_key_data -> printing_facilities
(** Printing facilities construction functions with static tag prefix.
 Similar to the printing facilities construction functions
 [*_printing_facilities] except that a static tag is prefixed to each
 printed message. This tag is the software name between square brackets
 characters (['['] and [']']). *)
;;

(** {6 Messages with dynamic tags} *)

type tag = string
and 'a make_tag_opt = 'a -> tag option
(** Dynamic tags are functions returning a string. *)
;;

type 'a dynamic_printing_facilities = {
  dyn_get_warn_error : unit -> bool;
  dyn_report : 'a -> message_print;
  dyn_debug : 'a -> message_gen_print;
  dyn_warning : 'a -> message_gen_print;
  dyn_error : 'a -> message_gen_print;
  dyn_report_action : 'a -> gen_message_print;
  dyn_debug_action : 'a -> gen_message_gen_print;
  dyn_warning_action : 'a -> gen_message_gen_print;
  dyn_error_action : 'a -> gen_message_gen_print;
}
(** The type of dynamic printing facilities.
 The fields are the similar to those of the type {!printing_facilities} except
 that they have an additional first argument to compute the dynamic tag. *)
;;

val application_printing_facilities_with_dynamic_tags :
  Ucommand_option.software_option_key_data -> 'a make_tag_opt ->
  'a dynamic_printing_facilities
;;
val library_printing_facilities_with_dynamic_tags :
  Ucommand_option.software_option_key_data -> 'a make_tag_opt ->
  'a dynamic_printing_facilities
(** Printing facilities construction functions with dynamic tag prefixes.
 Similar to the (static) printing facilities construction functions
 [*_printing_facilities_with_static_tag] except that the tag generator
 is given as an additional last argument. *)
;;

(** {6 Specializing printed messages} *)

type specialized_printing_facilities = {
  spf_get_warn_error : unit -> bool;
  spf_report : message_print;
  spf_debug : message_print;
  spf_warning : message_print;
  spf_error : message_print;
  spf_report_action : gen_message_print;
  spf_debug_action : gen_message_print;
  spf_warning_action : gen_message_print;
  spf_error_action : gen_message_print;
}
;;

val set_function_name :
  Upervasives.function_name ->
  printing_facilities -> specialized_printing_facilities
;;

val prefix_messages :
  Upervasives.prefix ->
  specialized_printing_facilities -> specialized_printing_facilities
;;

type 'a specialized_dynamic_printing_facilities = {
  sdpf_get_warn_error : unit -> bool;
  sdpf_report : 'a -> message_print;
  sdpf_debug : 'a -> message_print;
  sdpf_warning : 'a -> message_print;
  sdpf_error : 'a -> message_print;
  sdpf_report_action : 'a -> gen_message_print;
  sdpf_debug_action : 'a -> gen_message_print;
  sdpf_warning_action : 'a -> gen_message_print;
  sdpf_error_action : 'a -> gen_message_print;
}
;;

val set_function_name_dyn :
  Upervasives.function_name ->
  'a dynamic_printing_facilities ->
  'a specialized_dynamic_printing_facilities
;;

val prefix_messages_dyn :
  Upervasives.prefix ->
  'a specialized_dynamic_printing_facilities ->
  'a specialized_dynamic_printing_facilities
;;

(** {6 Managing automatic service options} *)

val warn_error_app_clear : unit -> unit;;
val warn_error_lib_clear : unit -> unit;;
val report_app_clear : unit -> unit;;
val report_lib_clear : unit -> unit;;
val debug_app_clear : unit -> unit;;
val debug_lib_clear : unit -> unit
(** Clear already added entries in the corresponding class of command
 options (see module {!Ucommand_option}). *)
;;

val warn_error_app_clear_option_key : Uoption_key.t -> unit;;
val warn_error_lib_clear_option_key : Uoption_key.t -> unit;;
val report_app_clear_option_key : Uoption_key.t -> unit;;
val report_lib_clear_option_key : Uoption_key.t -> unit;;
val debug_app_clear_option_key : Uoption_key.t -> unit;;
val debug_lib_clear_option_key : Uoption_key.t -> unit
(** Clear all already added entries in the corresponding class of command
 options which main option name matches the given [key] (see module
 {!Ucommand_option}). *)
;;

val phase_ulib_debugging : Ucommand_option.phase_name
(** Ulib debugging phase name. *)
;;

val phase_warn_error : unit -> unit;;
val phase_report : unit -> unit;;
val phase_debug : unit -> unit
(** Add all declared command options of the underlying class to the phase
 {!phase_ulib_debugging}, including automatically added service command
 option. *)
;;

val phase_all_debugging : unit -> unit
(** Shortcut to call all three [phase_*] functions above. *)
;;
