(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: udelayed_dgraph.ml,v 1.22 2017/05/31 15:24:06 fclement Exp $ *)

(* Delayed dynamic graphs. *)

module Dg = Udynamic_graph;;
module Ddgt = Udelayed_dgraph_types;;

let qual = Upervasives.qualify "Udelayed_dgraph";;
let invalid_arg = Upervasives.invalid_arg_qual qual;;

type arrow_number = Ddgt.arrow_number;;

type 'b delayed_image = 'b Ddgt.delayed_image =
  | Not_yet_computed
  | Image of 'b
;;

type ('a, 'b) delayed_arrow = ('a, 'b) Ddgt.delayed_arrow;;

type ('a, 'b, 'c) t = ('a, 'b, 'c) Ddgt.t = private
  {
   f : 'c -> arrow_number -> 'a -> 'b;
   graph : ('a, 'b delayed_image) Dg.t;
   to_do : arrow_number list;
  }
and ('a, 'b, 'c) delayed_dgraph = ('a, 'b, 'c) t
;;

(* Construction functions. *)

let make = Ddgt.make
and set_next = Ddgt.set_next
and set = Ddgt.set
and of_values = Ddgt.of_values
and do_it = Ddgt.do_it
;;

(* Operations on delayed dynamic graphs. *)

let get ddg = Dg.get ddg.graph;;

let same ddg1 ddg2 =
  ddg1.f == ddg2.f &&
  Dg.same ddg1.graph ddg2.graph &&
  ddg1.to_do = ddg2.to_do
;;

let equal_values equal_value ddg1 ddg2 =
  Dg.equal_values equal_value ddg1.graph ddg2.graph
;;

let get_image = function
  | Not_yet_computed -> assert false
  | Image x -> x
;;

let get_arrow _an ({ Dg.image = dim; _; } as ar) =
  { ar with Dg.image = get_image dim; }
;;

let unsafe_to_array ddg = Dg.to_array (Dg.map get_arrow ddg.graph);;

let check_nonempty_to_do fun_name f = function
  | { to_do = _ :: _; _; } -> invalid_arg fun_name "nonempty to_do list"
  | ({ to_do = []; _; } as ddg) -> f ddg
;;

let to_array ddg = check_nonempty_to_do "to_array" unsafe_to_array ddg;;

(* Iterators. *)

let unsafe_fold_left { graph = dg; _; } f accu =
  let ff accu i = function
    | { Dg.image = Not_yet_computed; _ } -> assert false
    | ({ Dg.image = Image _; _ } as ar) -> f accu i ar in
  Dg.fold_left ff accu dg
;;

let fold_left f accu ddg =
  check_nonempty_to_do "fold_left" unsafe_fold_left ddg f accu
;;

let iter f ddg =
  check_nonempty_to_do "iter" unsafe_fold_left ddg (fun _ i ar -> f i ar) ()
;;

let iter_to_do f ddg =
  let ff i =
    let ar = get ddg i in
    f i ar.Dg.value in
  Ulist.iter ff ddg.to_do
;;

let iter_value f ddg =
  let ff i ar = f i ar.Dg.value in
  Dg.iter ff ddg.graph
;;

let fold_left_value f accu ddg =
  let ff accu i ar = f accu i ar.Dg.value in
  Dg.fold_left ff accu ddg.graph
;;
