(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ufbuffer.mli,v 1.8 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 Flushing buffers}

 A flushing buffer is a character buffer with fixed length.
 When the buffer is full, it is automatically flushed via its user provided
 flushing method. *)

type length = Ubytes.length
and index = Ubytes.index
;;

type t
and fbuffer = t
(** Abstract type of flushing buffers. *)
;;

val create : (string -> unit) -> length -> t
(** [Ufbuffer.create flush n] returns a fresh flushing buffer, initially empty.
 The [n] parameter is the size of the internal byte sequence that holds the
 buffer contents.
 When more than [n] characters are stored in the buffer, the actual contents
 of the buffer is passed to function [flush] for flushing and the buffer is
 empty again.
 For instance, flushing may occur on an output channel.
 For best performance [n] should not be too small (at least greater than the
 number of characters of a few typical string additions to the buffer).
 If the initial size [n] is not between 1 and [Sys.max_string_length], it will
 be clipped to that interval. *)
;;

val add_char : t -> char -> unit
(** [Ufbuffer.add_char b c] appends character [c] at the end of buffer [b]. *)
;;

val add_string : t -> string -> unit
(** [Ufbuffer.add_string b s] appends string [s] at the end of buffer [b]. *)
;;

val add_bytes : t -> Upervasives.bytes -> unit
(** [Ufbuffer.add_bytes b s] appends byte sequence [s] at the end of buffer [b]. *)
;;

val flush : t -> unit
(** [Ufbuffer.flush b] flushes buffer [b] as if [b] were full.
  Nothing happend if [b] is already empty. *)
;;
