(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix.mli,v 1.43 2017/10/30 18:38:49 fclement Exp $ *)

(** {3 Operations on matrices}

 A matrix is a two-dimensional array, or equivalently an array of values
 such that each value is an array of a fixed length.
 An element in a matrix is indexed by a pair of natural numbers and accessed
 in first-index-major order, that is, " la C" or using the lexicographic
 ordering for pairs of integers.
 The only matrix with no element is the empty matrix [\[||\]].

 Following the classical mathematical convention, the first index to access
 an element refers to the row number and is denoted by [i], and conversely
 the second index refers to the column number and is denoted by [j].
 As usual, the row numbers are increasing from top to bottom and the column
 numbers are increasing from left to right.

 Beware: these notions are conventional and subject to modification
 according to the problem at hand.
 For instance, in graphical applications the row numbers may decrease from
 top to bottom. *)

type 'a t = 'a Umatrix_types.t
and 'a matrix = 'a t
(** The type of matrices. *)
;;

val of_array_array : 'a array array -> 'a t
(** Convert the given array of arrays into a matrix. *)
;;

external to_array_array : 'a t -> 'a array array = "%identity"
(** Convert the given matrix into an array of arrays. *)
;;

type length = Uarray.length
and index = Uarray.index
and offset = Uarray.offset

and line_number = length
and column_number = length
and line_index = index
and column_index = index
and cell_index = index
and line_offset = offset
and column_offset = offset
(** Aliases for the type of natural numbers. *)
;;

type size = line_number * column_number
and index2 = line_index * column_index
and cell = index2
and offset2 = line_offset * column_offset
(** Aliases for the type of couples of natural numbers. *)
;;

val size : 'a t -> size
(** Return the size of the given matrix.
 The size of a matrix is a pair of naturals that gives the number of elements
 in both directions. *)
;;

val length : 'a t -> length
(** Return the length of the given matrix.
 The length of a matrix is its total number of elements. *)
;;

val dim_i : 'a t -> line_number
(** [dim_i m] returns the first dimension of the matrix [m].
 Hence, the first index of [m] varies from [0] to {!dim_i} [m - 1]. *)
;;

val dim_j : 'a t -> column_number
(** [dim_j m] returns the second dimension of the matrix [m].
 Hence, the second index of [m] varies from [0] to {!dim_j} [m - 1]. *)

val get : 'a t -> line_index -> column_index -> 'a
(** [get m i j] returns the element number [(i, j)] of matrix [m].

 Raise [Invalid_argument] if [(i, j)] is not a valid cell index for [m],
 i.e. if [i] is not a valid line index for [m], or if [j] is not a valid
 column for [m], i.e. if [i] is outside range [0] to {!dim_i} [m - 1],
 or [j] is outside range [0] to {!dim_j} [m - 1]. *)
;;

val get2 : 'a t -> cell -> 'a
(** [get2 m (i, j)] is equivalent to [get m i j]. *)
;;

val get_line : 'a t -> line_index -> 'a array
(** [get_line m i] returns a fresh array containing the elements of the line
 number [i].

 Raise [Invalid_argument] if [i] is not a valid line index for matrix [m]
 (see {!get}). *)
;;

val get_column : 'a t -> column_index -> 'a array
(** [get_column m j] returns a fresh array containing the elements of the
 column number [j].

 Raise [Invalid_argument] if [j] is not a valid column index for matrix [m]
 (see {!get}). *)
;;

val set : 'a t -> line_index -> column_index -> 'a -> unit
(** [set m i j x] modifies matrix [m] in place, replacing element number
 [(i, j)] with [x].

 Raise [Invalid_argument], if [(i, j)] is not a valid cell index for [m]
 (see {!get}. *)
;;

val set2 : 'a t -> cell -> 'a -> unit
(** [set2 m (i, j) x] is equivalent to [set m i j x]. *)
;;

val set_line : 'a t -> line_index -> 'a array -> unit
(** [set_line m i ai] modifies matrix [m] in place, replacing line number [i]
 with array [ai].

 Raise [Invalid_argument] if [i] is not a valid line index for matrix [m]
 (see {!get}), or if array [ai] is not of length {!dim_j} [m]. *)
;;

val set_column : 'a t -> column_index -> 'a array -> unit
(** [set_column m j aj] modifies matrix [m] in place, replacing elements of
 the column number [j] by those of array [aj].

 Raise [Invalid_argument] if [j] is not a valid column index for matrix [m]
 (see {!get}), or if array [aj] is not of length {!dim_i} [m]. *)
;;

val empty : 'a t
(** The empty matrix. *)
;;

val make : line_number -> column_number -> 'a -> 'a t
(** [make ni nj x] returns a fresh matrix with size [(ni, nj)].
 All the elements of this new matrix are initially physically equal to [x].

 Raise [Invalid_argument], if [ni] or [nj] is greater than
 [Sys.max_array_length].
 If the type of [x] is [float], then the maximum line and column numbers are
 only [Sys.max_array_length / 2]. *)
;;

val make2 : size -> 'a -> 'a t
(** [make2 (ni, nj) x] is equivalent to [make ni nj x]. *)
;;

val init :
  line_number -> column_number -> (line_index -> column_index -> 'a) -> 'a t
(** [init ni nj f] returns a fresh matrix of size [(ni, nj)], with element
 number [(i, j)] initialized to the result of [f i j].
 In other terms, it tabulates the results of [f] applied to the pairs of
 integers [(0, 0)] to [(ni - 1, nj - 1)].

 Raise [Invalid_argument], if [ni] or [nj] is greater than
 [Sys.max_array_length].
 If the return type of [f] is [float], then the maximum line and column
 numbers are only [Sys.max_array_length / 2]. *)
;;

val init2 : size -> (cell -> 'a) -> 'a t
(** [init2 (ni, nj) f] is equivalent to [init ni nj (fun i j -> f (i, j))]. *)
;;

val append_i : 'a t -> 'a t -> 'a t
(** [append_i m1 m2] returns a fresh matrix containing the concatenation of
 matrices [m1] and [m2] along the first direction.

 Raise [Invalid_argument], if the two matrices have different second
 dimensions. *)
;;

val append_j : 'a t -> 'a t -> 'a t
(** [append_j m1 m2] returns a fresh matrix containing the concatenation of
 matrices [m1] and [m2] along the second direction.

 Raise [Invalid_argument], if the two matrices have different first
 dimensions. *)
;;

val append : 'a t -> 'a t -> 'a t -> 'a t -> 'a t
(** [append m1 m2 m3 m4] returns a fresh matrix containing the concatenation
 of matrices [m1], [m2], [m3] and [m4].
 It is equivalent to
 {!append_i} [(]{!append_j} [m1 m2) (]{!append_j} [m3 m4)]. *)
;;

val concat_i : 'a t list -> 'a t
(** Same as {!append_i}, but concatenates a list of matrices. *)
;;

val concat_j : 'a t list -> 'a t
(** Same as {!append_j}, but concatenates a list of matrices. *)
;;

val concat : 'a t list list -> 'a t
(** [concat mll] returns a fresh matrix containing the concatenation of
 matrices of the list of lists [mll]. *)
;;

val sub :
  'a t -> line_offset -> line_number ->
   column_offset -> column_number -> 'a t
(** [sub m oi li oj lj] returns a fresh matrix of size [(li, lj)], containing
 the elements number [(oi, oj)] to [(oi + li - 1, oj + lj - 1)] of matrix [m].

 Raise [Invalid_argument], if [oi], [li], [oj], and [lj] do not designate a
 valid submatrix of [m]; that is, if [oi + li >] {!dim_i} [m], or
 [oj + lj >] {!dim_j} [m]. *)
;;

val sub2 : 'a t -> offset2 -> size -> 'a t
(** [sub2 m (oi, oj) (li, lj)] is equivalent to [sub m oi li oj lj]. *)
;;

val copy : 'a t -> 'a t
(** [copy m] returns a copy of [m], that is, a fresh matrix containing the
 same elements as [m]. *)
;;

val for_loop2 : offset2 -> size -> (cell -> unit) -> unit
(** [for_loop2 (oi, oj) (li, lj) f] is equivalent to
 {!Uarray.for_loop} [oi li (fun i -> ]
 {!Uarray.for_loop} [oj lj (fun j -> f (i, j))]. *)
;;

val nat_for_loop2 : cell -> cell -> (cell -> unit) -> unit
(** [nat_for_loop2 (ifirst, jfirst) (ilast, jlast) f] is equivalent to
 {!Uarray.nat_for_loop} [ifirst ilast (fun i -> ]
 {!Uarray.nat_for_loop} [jfirst jlast (fun j -> f (i, j))]. *)
;;

val nat_rev_for_loop_i2 : cell -> cell -> (cell -> unit) -> unit
(** [nat_rev_for_loop_i2 (ifirst, jfirst) (ilast, jlast) f] is equivalent to
 {!Uarray.nat_rev_for_loop} [ifirst ilast (fun i -> ]
 {!Uarray.nat_for_loop} [jfirst jlast (fun j -> f (i, j))]. *)
;;

val nat_rev_for_loop_j2 : cell -> cell -> (cell -> unit) -> unit
(** [nat_rev_for_loop_j2 (ifirst, jfirst) (ilast, jlast) f] is equivalent to
 {!Uarray.nat_for_loop} [ifirst ilast (fun i -> ]
 {!Uarray.nat_rev_for_loop} [jfirst jlast (fun j -> f (i, j))]. *)
;;

val nat_rev_for_loop_ij2 : cell -> cell -> (cell -> unit) -> unit
(** [nat_rev_for_loop_ij2 (ifirst, jfirst) (ilast, jlast) f] is equivalent to
 {!Uarray.nat_rev_for_loop} [ifirst ilast (fun i -> ]
 {!Uarray.nat_rev_for_loop} [jfirst jlast (fun j -> f (i, j))]. *)
;;

val fill :
  'a t -> line_offset -> line_number ->
  column_offset -> column_number -> 'a -> unit
(** [fill m oi li oj lj x] modifies the matrix [m] in place, storing [x] in
 elements number [(oi, oj)] to [(oi + li - 1, oj + lj - 1)].

 Raise [Invalid_argument], if [oi], [li], [oj] and [lj] do not designate a
 valid submatrix of [m] (see {!sub}). *)
;;

val fill2 : 'a t -> offset2 -> size -> 'a -> unit
(** [fill2 m (oi, oj) (li, lj) x] is equivalent to [fill m oi li oj lj x]. *)
;;

val blit :
  'a t -> line_offset -> column_offset ->
  'a t -> line_offset -> column_offset ->
  line_number -> column_number -> unit
(** [blit m1 oi1 oj1 m2 oi2 oj2 li lj] copies [li * lj] elements from matrix
 [m1], starting at element number [(oi1, oj1)], to matrix [m2], starting at
 element number [(oi2, oj2)].
 It works correctly even if [m1] and [m2] are the same matrix, and the source
 and destination chunks overlap.

 Raise [Invalid_argument], if [oi1], [li], [oj1] and [lj] do not designate
 a valid submatrix of [m1], or if [oi2], [li], [oj2] and [lj] do not
 designate a valid submatrix of [m2] (see {!sub}). *)
;;

val blit2 : 'a t -> offset2 -> 'a t -> offset2 -> size -> unit
(** [blit2 m1 (oi1, oj1) m2 (oi2, oj2) (li, lj)] is equivalent to
 [blit m oi1 oj1 m oi2 oj2 li lj]. *)
;;

val transpose : 'a t -> 'a t
(** [transpose m] returns the transposed of matrix [m], that is, a fresh
 matrix of size [(]{!dim_j} [m,] {!dim_i} [m)] whose element number [(i, j)]
 is equal to [m.(j).(i)].

 Transposition is an involutive operation, since array arrays of size
 [(ni, 0)] with [ni > 0] are not valid matrices. *)
;;

val is_empty : 'a t -> bool
(** [is_empty m] is true if and only if the matrix [m] is empty. *)
;;

val check_nonempty : Upervasives.function_name -> 'a t -> unit
(** Check if the given matrix is nonempty.

 Raise [Invalid_argument] with the argument function name when the matrix is
 empty. *)
;;

(** {6 Converters} *)

val cell_index : column_number -> cell -> cell_index
(** [cell_index nj (i, j)] returns the cell index of cell [(i, j)], which is
 [nj * i + j]. *)
;;

val cell : column_number -> cell_index -> cell
(** [cell nj ii] returns the cell with cell index [ii], which is
 [(ii / nj, ii mod nj]. *)
;;

val to_array : 'a t -> 'a array
(** [to_array m] returns a fresh array containing the concatenation of all
 subarrays along the first direction of matrix [m].
 It is equivalent to
 {!Uarray.concat} [\[| m.(0); m.(1); ...; m.(ni - 1) |\]] where [ni] is the
 first dimension of matrix [m]. *)
;;

val of_array : 'a array -> line_number -> column_number -> 'a t
(** [of_array a ni nj] returns a fresh matrix containing the split of array
 [a] into [ni] subarrays of length [nj].

 Raise [Invalid_argument], if array [a] is not of the length [ni * nj]. *)
;;

val of_array2 : 'a array -> size -> 'a t
(** [of_array2 a (ni, nj)] is equivalent to [of_array a ni nj]. *)
;;

val to_array_list : 'a t -> 'a array list
(** [to_array_list m] returns the list of the arrays along the first direction
 of matrix [m].
 It is equivalent to [\[| m.(0); m.(1); ...; m.(ni) |\]] where [ni] is the
 first dimension of matrix [m]. *)
;;

val of_array_list : 'a array list -> 'a t
(** [of_array_list al] returns a fresh matrix containing the array of arrays
 list [al].

 Raise [Invalid_argument], if all the arrays of list [al] do not have the
 same length. *)
;;

val to_list : 'a t -> 'a list list
(** [to_list m] returns the list of lists of the elements of [m]. *)
;;

val of_list : 'a list list -> 'a t
(** [of_list l] returns a fresh matrix containing the elements of the list of
 lists [l]. *)
;;

(** {6 Iterators} *)

val iter : ('a -> unit) -> 'a t -> unit
(** [iter f m] applies function [f] in turn to all the elements of [m].
 It is equivalent to
 {!Uarray.iter} [f m.(0);] {!Uarray.iter} [f m.(1); ...;]
 {!Uarray.iter} [f m.(ni - 1); ()] where [ni] is the first dimension of
 matrix [m]. *)
;;

val map : ('a -> 'b) -> 'a t -> 'b t
(** [map f m] applies function [f] to all the elements of [m], and builds a
 matrix with the results returned by [f]:
 [\[|] {!Uarray.map} [f m.(0);] {!Uarray.map} [f m.(1); ...;]
 {!Uarray.map} [f a.(ni - 1) |\]] where [ni] is the first dimension of
 matrix [m]. *)
;;

val iter_ij : (line_index -> column_index -> 'a -> unit) -> 'a t -> unit
(** Same as {!iter}, but the function is applied to the indices of the element
 as first and second arguments, and the element itself as third argument. *)

val iter_ij2 : (cell -> 'a -> unit) -> 'a t -> unit
(** [iter_ij2 f m] is equivalent to [iter_ij (fun i j  x -> f (i,j) x) m]. *)
;;

val map_ij : (line_index -> column_index -> 'a -> 'b) -> 'a t -> 'b t
(** Same as {!map}, but the function is applied to the indices of the element
 as first and second arguments, and the element itself as third argument. *)
;;

val map_ij2 : (cell -> 'a -> 'b) -> 'a t -> 'b t
(** [map_ij2 f m] is equivalent to [map_ij (fun i j x -> f (i, j) x) m]. *)
;;

val iter_i : ('a array -> unit) -> 'a t -> unit
(** [iter_i f m] applies function [f] in turn to all the rows of [m].
 It is equivalent to {!Uarray.iter} [f m]. *)
;;

val iter_j : ('a array -> unit) -> 'a t -> unit
(** Same as {!iter_i}, but applied to the columns of the matrix argument.
 It is equivalent to {!iter_i} [f (]{!transpose} [m)]. *)
;;

val map_i : ('a array -> 'b array) -> 'a t -> 'b t
(** [map_i f m] applies function [f] in turn to all the rows of [m].
 It is equivalent to {!Uarray.map} [f m]. *)
;;

val map_j : ('a array -> 'b array) -> 'a t -> 'b t
(** Same as {!map_i}, but applied to the columns of the matrix.
 It is equivalent to {!transpose} [(]{!map_i} [f (]{!transpose} [m))]. *)
;;

val fold_left : ('a -> 'b -> 'a) -> 'a -> 'b t -> 'a
(** [fold_left f x m] computes
 {!Uarray.fold_left} [f (... (] {!Uarray.fold_left} [f (]
 {!Uarray.fold_left} [f x m.(0)) m.(1)) ...) m.(ni - 1)],
 where [ni] is the first dimension of matrix [m]. *)
;;

val fold_right : ('a -> 'b -> 'b) -> 'a t -> 'b -> 'b
(** [fold_right f m x] computes
 {!Uarray.fold_right} [f m.(0) (] {!Uarray.fold_right} [f m.(1) (... (]
 {!Uarray.fold_right} [f m.(ni - 1) x) ...))], where [ni] is the first
 dimension of matrix [m]. *)
;;

val sub_iter :
  ('a -> unit) -> 'a t -> line_offset -> line_number ->
  column_offset -> column_number -> unit
(** [sub_iter f m oi li oj lj] applies function [f] in turn to the elements
 number [(oi, oj)] to [(oi + li - 1, oj + lj - 1)] of matrix [m].
 It is equivalent to {!Uarray.sub_iter} [f m.(oi) oj lj;]
 {!Uarray.sub_iter} [f m.(oi + 1) oj lj; ...;]
 {!Uarray.sub_iter} [f m.(oi + li - 1) oj lj; ()]. *)
;;

val sub_iter2 :
  ('a -> unit) -> 'a t -> offset2 -> size -> unit
(** [sub_iter2 f m (oi, oj) (li, lj)] is equivalent to
 [sub_iter f m oi li oj lj]. *)
;;

val sub_iter_ij :
  (line_index -> column_index -> 'a -> unit) -> 'a t ->
  line_offset -> line_number -> column_offset -> column_number -> unit
(** Same as {!sub_iter}, but the function is applied to the indices of the
 element as first and second arguments, and the element itself as third
 argument. *)
;;

val sub_iter_ij2 : (cell -> 'a -> unit) -> 'a t -> offset2 -> size -> unit
(** [sub_iter_ij2 f m (oi, oj) (li, lj)] is equivalent to
 [sub_iter_ij (fun i j x -> f (i, j) x) m oi li oj lj]. *)
;;

(** {6 Iterators on two matrices} *)

val iter2 : ('a -> 'b -> unit) -> 'a t -> 'b t -> unit
(** [iter2 f m1 m2] applies function [f] in turn to all the elements of [m1]
 and [m2].
 It is equivalent to {!Uarray.iter2} [f m1.(0) m2.(0);]
 {!Uarray.iter2} [f m1.(1) m2.(1); ...;]
 {!Uarray.iter2} [f m1.(ni - 1) m2.(ni - 1); ()], where [ni] is the first
 dimension of matrices [m1] and [m2].

 Raise [Invalid_argument] if both matrices do not have the same size. *)
;;

val map2 : ('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t
(** [map2 f m1 m2] applies function [f] to all the elements of [m1] and [m2],
 and builds a matrix with the results returned by [f]:
 [\[|] {!Uarray.map2} [f m1.(0) m2.(0);] {!Uarray.map2} [f m1.(1) m2.(1);
 ...;] {!Uarray.map2} [f m1.(ni - 1) m2.(ni - 1) |\]], where [ni] is the
 first dimension of matrices [m1] and [m2].

 Raise [Invalid_argument] if both matrices do not have the same size. *)
;;

val iter2_ij :
  (line_index -> column_index -> 'a -> 'b -> unit) ->
  'a t -> 'b t -> unit
(** Same as {!iter2}, but the function is applied to the indices of the
 elements as first and second arguments, and the elements themselves as
 third and fourth arguments. *)
;;

val iter2_ij2 : (cell -> 'a -> 'b -> unit) -> 'a t -> 'b t -> unit
(** [iter2_ij2 f m1 m2] is equivalent to
 [iter2_ij (fun i j x -> f (i, j) x) m1 m2]. *)
;;

val map2_ij :
  (line_index -> column_index -> 'a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t
(** Same as {!map2}, but the function is applied to the indices of the
 elements as first and second arguments, and the elements themselves as
 third and fourth arguments. *)
;;

val map2_ij2 : (cell -> 'a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t
(** [map2_ij2 f m1 m2] is equivalent to
 [map2_ij (fun i j x -> f (i, j) x) m1 m2]. *)
;;

val fold_left2 :
  ('a -> 'b -> 'c -> 'a) -> 'a -> 'b t -> 'c t -> 'a
(** [fold_left2 f x m1 m2] computes
 {!Uarray.fold_left2} [f (... (]{!Uarray.fold_left2} [f (]
 {!Uarray.fold_left2} [f x m1.(0) m2.(0)) m1.(1) m2.(1)) ...)
 m1.(ni - 1) m2.(ni - 1)], where [ni] is the first dimension of the matrices
 [m1] and [m2].

 Raise [Invalid_argument] if both matrices do not have the same size. *)
;;

val fold_right2 :
  ('a -> 'b -> 'c -> 'c) -> 'a t -> 'b t -> 'c -> 'c
(** [fold_right2 f m1 m2 x] computes
 {!Uarray.fold_right2} [f m1.(0) m2.(0) (]
 {!Uarray.fold_right2} [f m1.(1) m2.(1) (... (]
 {!Uarray.fold_right2} [f m1.(ni - 1) m2.(ni - 1) x) ...))], where [ni] is
 the first dimension of matrices [m1] and [m2].

 Raise [Invalid_argument] if both matrices do not have the same size. *)
;;

(** {6 Matrix scanning} *)

val for_all : ('a -> bool) -> 'a t -> bool
(** [for_all p m] checks if all elements of the matrix satisfy the predicate
 [p].
 That is, it returns [p m.(0).(0) && ... && p m.(0).(nj-1) && ... &&
 p m.(ni-1).(0) && ... && p m.(ni-1).(nj-1)]. *)
;;

val exists : ('a -> bool) -> 'a t -> bool
(** [exists p m] checks if at least one element of the matrix satisfies the
 predicate [p].
 That is, it returns [p m.(0).(0) || ... || p m.(0).(nj-1) || ... ||
 p m.(ni-1).(0) || ... || p m.(ni-1).(nj-1)]. *)
;;

val mem : 'a -> 'a t -> bool
(** [mem x m] is true if and only if [x] is equal to an element of [m]. *)
;;

val memq : 'a -> 'a t -> bool
(** Same as {!mem}, but uses physical equality instead of structural equality
 to compare array elements. *)
;;

(** {6 Selecting elements in matrices} *)

val select : Upervasives.select t -> 'a t -> 'a array array
(** [select chi m] returns a fresh array array containing elements
 [m.(i).(j)] such that [chi.(i).(j)] is {!Upervasives}.[Keep]. The cell order
 is preserved.

 Raise [Invalid_argument] if the two matrices have different sizes. *)
;;

val map_select : ('a -> 'b) -> Upervasives.select t -> 'a t -> 'b array array
(** [select f chi m] returns a fresh array array containing elements
 [f m.(i).(j)] such that [chi.(i).(j)] is {!Upervasives}.[Keep]. The cell order
 is preserved.

 Raise [Invalid_argument] if the two matrices have different sizes. *)
;;

val unselect : 'a -> Upervasives.select t -> 'a array array -> 'a t
(** [unselect x chi aa] returns a fresh matrix of the same size as [chi]
 such that its[j]-th element of [i]-th line is [x] when [chi.(i).(j) =]
 {!Upervasives}[.Forget] and [aa.(i).(k)] when [chi.(i).(j)] is the [k]-th
 {!Upervasives}[.Keep] value in line [chi.(i)].

 For all [x], [select chi (unselect x chi aa)] gives back an array array
 equal to [aa].

 Raise [Failure] if [chi] and [aa] have different number of lines, or if
 there exists a line number [i] such that the number of {!Upervasives}[.Keep]
 values in [chi.(i)] is different from the length of [aa.(i)]. *)
;;

(** {6 Specific applications} *)

val make_diagonal_matrix : 'a -> 'a array -> 'a t
(** [make_diagonal_matrix zero v] returns a fresh diagonal matrix with its
 main diagonal filled with elements of the vector [v] and all other entries
 filled with [zero]. *)
;;

val make_int_diagonal_matrix : int array -> int matrix;;
val make_float_diagonal_matrix : float array -> float matrix
(** Specializations of {!make_diagonal_matrix} for integers and floats with the
 canonical zero value. *)

val resize : line_number -> column_number -> 'a t -> 'a t
(** [resize ni nj m] returns a fresh matrix of size [(ni, nj)].
 Each element of the given matrix [m] becomes a submatrix of size ...

 Raise [Invalid_arg] if [ni = 0] and [nj <> 0], or if the matrix [m] is not
 of length [ni * nj]. *)
;;

val resize2 : size -> 'a t -> 'a t
(** [resize2 (ni, nj) m] is equivalent to [resize ni nj m]. *)
;;

val zoom : line_number -> column_number -> 'a t -> 'a t
(** [zoom ni nj m] returns a fresh matrix whose dimensions are those of [m]
 multiplied by [ni] and [nj].
 Each element of the given matrix [m] becomes a submatrix of size [(ni, nj)]. *)
;;

val zoom2 : size -> 'a t -> 'a t
(** [zoom2 (ni, nj) m] is equivalent to [zoom ni nj m]. *)
;;

(**/**)
(** {6 Undocumented features} *)

external unsafe_of_array_array : 'a array array -> 'a t = "%identity";;

val unsafe_get : 'a t -> line_index -> column_index -> 'a;;
val unsafe_set : 'a t -> line_index -> column_index -> 'a -> unit;;
val unsafe_get2 : 'a t -> cell -> 'a;;
val unsafe_set2 : 'a t -> cell -> 'a -> unit;;
