(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: unatural.mli,v 1.31 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 Natural numbers} *)

type t = Unatural_types.t
and natural = t
(** The type of natural numbers. *)
;;

val _0_ : t;;
val _1_ : t;;
val _2_ : t;;
val _3_ : t;;
val _4_ : t;;
val _5_ : t;;
val _6_ : t;;
val _7_ : t;;
val _8_ : t;;
val _9_ : t
(** The naturals 0 to 9. *)
;;

val add : t -> t -> t
(** Addition. See {!mul}. *)
;;

val sub : t -> t -> t
(** Subtraction. See {!mul}. *)
;;

val mul : t -> t -> t
(** Multiplication.

 Raise [Failure] when the result is a negative integer. *)
;;

val div : t -> t -> t
(** Integer division. This division rounds the real quotient of its arguments
 towards zero, as specified for [Pervasives.( / )].

 Raise [Division_by_zero] if the second argument is {!_0_}. *)
;;

val rem : t -> t -> t
(** Integer remainder. If [y] is not {!_0_}, the result of [rem x y]
 satisfies the following property:
 [x =] {!add} [(]{!mul} [(]{!div} [x y) y) (]{!rem} [x y)].

 Raise [Division_by_zero] if the second argument is {!_0_}. *)
;;

val gcd : t -> t -> t
(** Greatest common divisor. *)
;;

val succ : t -> t
(** Successor. [succ x] is equivalent to {!add} [x] {!_1_}. *)
;;

val pred : t -> t
(** Predecessor. [pred x] is equivalent to {!sub} [x] {!_1_}. *)
;;

val max_nat : t
(** The greatest representable natural, {!of_int} [Pervasives.max_int]. *)
;;

val min_nat : t
(** The smallest representable natural, {!_0_}. *)
;;

val of_int : int -> t
(** Convert the given integer into a natural. *)
;;

external to_int : t -> int = "%identity"
(** Convert the given natural into an integer. *)
;;

val of_float : float -> t
(** Convert the given floating-point number to a natural, discarding the
 fractional part (round towards zero). The result of the conversion is
 undefined if, after truncation, the number is outside the range
 \[{!min_nat}..{!max_nat}\]. *)
;;

val to_float : t -> float
(** Convert the given natural to a floating-point number. *)
;;

external of_char : char -> t = "%identity"
(** Return the ASCII code of the argument. *)
;;

val to_char : t -> char
(** Return the character with the given ASCII code.

 Raise [Invalid_argument] if the argument is outside the range \[0..255\]. *)
;;

val of_string : string -> t
(** Convert the given string to a natural. The string is read in decimal
 (by default) or in hexadecimal, octal or binary if it begins with ["0x"],
 ["0o"] or ["0b"] respectively.

 Raise [Failure] if the given string is not a valid representation of an
 integer, or if the integer represented is outside of the range of
 representable naturals \[{!min_nat}..{!max_nat}\]. *)
;;

val to_string : t -> string
(** Return the string representation of its argument, in decimal. *)
;;

val compare : t -> t -> int
(** The comparison function for naturals, with the same specification as
 [Pervasives.compare]. Along with the type [t], this function [compare]
 allows the module [Unatural] to be passed as argument to the functors
 [Set.Make] and [Map.Make]. *)
;;

val split : t -> t -> t list
(** [split n k] returns a list of [k] integers that differs at most by 1
 and such that their sum equals [n]. *)
;;

val psplit : t -> t -> t list
(** Same as [split] but the result is a list of at most [k] positive
 integers. *)
;;

val incr : t ref -> unit
(** Increment. [incr r] is [r :=] {!succ} [!r]. *)
;;

val decr : t ref -> unit
(** Decrement. [decr r] is [r :=] {!pred} [!r].*)
;;

val is_zero : t -> bool;;
val is_positive : t -> bool;;
val is_even : t -> bool;;
val is_odd : t -> bool
(** Predicates on natural numbers. *)
;;

type tmin = t
and tmax = t
;;

val for_loop : tmin -> tmax -> (t -> unit) -> unit
(** [for_loop nfirst nlast body] is equivalent to
 [for n =] {!to_int} [nfirst to] {!to_int} [nlast do
 body (]{!of_int} [n) done]. *)
;;

val rev_for_loop : tmax -> tmin -> (t -> unit) -> unit
(** [rev_for_loop nfirst nlast body] is equivalent to
 [for n =] {!to_int} [nfirst downto] {!to_int} [nlast do
 body (]{!of_int} [n) done]. *)
;;

(**/**)
(** {6 Undocumented features} *)

external unsafe_of_int : int -> t = "%identity";;
external unsafe_to_char : t -> char = "%identity";;
