(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: unonempty_set_types.ml,v 1.7 2017/05/31 15:24:06 fclement Exp $ *)

(** The private type of nonempty sets. *)

module Nat = Unatural;;
module L1 = Ulist1;;
module A = Uarray;;
module Univ = Uuniverse;;

let _0_ = Nat._0_;;

let qual = Upervasives.qualify "Unonempty_set";;
let invalid_arg = Upervasives.invalid_arg_qual qual
and ucheck_index fun_name = Univ.check_index (qual fun_name)
;;

type 'a uindex = 'a Univ.index;;

type 'a index = 'a uindex uindex;;

type 'a t =
  {
   universe : 'a Univ.t;
   uindices : 'a uindex Univ.t;
  }
and 'a nonempty_set = 'a t
;;

let of_array u uindices =
  let len = A.length uindices in
  if len = _0_ then invalid_arg "of_array" "empty array" else begin
  (* Now, we have len > 0.
     Therefore, the following Nat.pred will not fail. *)
    A.sort Pervasives.compare uindices;
    let last_uindex = A.get uindices (Nat.pred len) in
    ucheck_index "of_array" u last_uindex;
    { universe = u; uindices = Univ.of_array uindices; }
  end
;;

let full u = { universe = u; uindices = Univ.indices u; };;

let unsafe_of_list u uindices =
  { universe = u; uindices = Univ.of_list1 uindices; }
;;

let of_list u uindices =
  let uindices = L1.sort Pervasives.compare uindices in
  let last_uindex = L1.last uindices in
  ucheck_index "of_list" u last_uindex;
  unsafe_of_list u uindices
;;

let same_universe nes1 nes2 = nes1.universe == nes2.universe;;

let check_same_universe fun_name nes1 nes2 =
  if not (same_universe nes1 nes2) then invalid_arg fun_name "universe clash"
;;

let cardinal nes = Univ.cardinal nes.uindices;;

let fold_left f accu nes = Univ.fold_left f accu nes.uindices;;

let union nes1 nes2 =
  check_same_universe "union" nes1 nes2;
  let (smaller, greater) =
    if cardinal nes1 < cardinal nes2 then (nes1, nes2) else (nes2, nes1) in
  let uindices =
    fold_left
      (fun accu i -> Univ.add accu i; accu)
      (Univ.copy greater.uindices) smaller in
  { greater with uindices = uindices; }
;;
