(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: upervasives.ml,v 1.52 2017/10/27 10:43:36 fclement Exp $ *)

(* Extensions to the initially opened module. *)

module Nat = Unatural;;

(* Byte sequences. *)

type bytes = Upervasives_types.bytes;;

(* Functional helpers. *)

external id : 'a -> 'a = "%identity";;

let compose g f x = g (f x);;

let proj2_1 x _ = x
and proj2_2 _ y = y
;;

let liftl_option d f = function
  | None -> d
  | Some x -> f x
and liftr_option f x = Some (f x)
;;

let exercise_option d = liftl_option d id;;

let downl_option f x = f (Some x)
and downr_option d f x = exercise_option d (f x);;

let lift_option f = liftl_option None (liftr_option f)
and down_option d f = downl_option (downr_option d f)
                 (* = downr_option d (downl_option f) *)
;;

let liftl_option2 d f x_opt y_opt =
  match (x_opt, y_opt) with
  | (None, _) | (_, None) -> d
  | (Some x, Some y) -> f x y
and liftr_option2 f x y = Some (f x y)
;;

let downl_option2 f x y = downl_option (downl_option f x) y
and downr_option2 d f x y = exercise_option d (f x y);;

let _lift_option2 f = liftl_option2 None (liftr_option2 f)
and down_option2 d f = downl_option2 (downr_option2 d f)
                  (* = downr_option2 d (downl_option2 f) *)
;;

(* Generating messages. *)

type software_name = string
and module_name = string
and function_name = string
and tag = string
and prefix = string
and message = string

and gen_message = unit -> message
and gen_message_option = unit -> message option
and qualify = function_name -> function_name
and qualify_opt = function_name option -> function_name option
;;

let uncautious_combine sep s s' = Printf.sprintf "%s%s%s" s sep s';;

let combine sep s_opt s_opt' =
  match (s_opt, s_opt') with
  | (None, None) -> None
  | (None, Some s) | (Some s, None) -> Some s
  | (Some s, Some s') -> Some (uncautious_combine sep s s')

and cautious_combine sep s_opt s_opt' =
  match (s_opt, s_opt') with
  | (None, _) -> s_opt'
  | (Some _, None) -> None
  | (Some s, Some s') -> Some (uncautious_combine sep s s')
;;

let prefix_opt = combine " "
and cautious_prefix_opt = cautious_combine " "
and qualify_opt = combine "."
and quote_opt = combine ": "
;;

let down_string_option2 f = down_option2 "" f;;

let prefix = down_string_option2 prefix_opt
and qualify = down_string_option2 qualify_opt
and quote = down_string_option2 quote_opt
;;

let prefix_and_quote_opt tag_opt fun_name_opt gen_msg_opt =
  prefix_opt tag_opt (quote_opt fun_name_opt (gen_msg_opt ()))
;;

let prefix_quote_and_prefix_opt ann_opt tag_opt fun_name_opt =
  prefix_and_quote_opt tag_opt
    (prefix_opt ann_opt
       (cautious_prefix_opt (Some "in") fun_name_opt))
;;

let report_message_opt tag_opt gen_msg_opt = prefix_opt tag_opt (gen_msg_opt ())
and debug_message_opt = prefix_and_quote_opt
and warning_message_opt = prefix_quote_and_prefix_opt (Some "Warning")
and error_message_opt = prefix_quote_and_prefix_opt (Some "Fatal error")
;;

let gen_opt x = fun () -> Some x;;

let downr_string_option f = downr_option "" f;;

let down_gm2 f tag msg = downr_string_option (f tag) (gen_opt msg)
and down_gm3 f tag fun_name msg =
  downr_string_option (f tag fun_name) (gen_opt msg)
;;

let report_message = down_gm2 report_message_opt
and debug_message = down_gm3 debug_message_opt
and warning_message = down_gm3 warning_message_opt
and error_message = down_gm3 error_message_opt
;;

(* Raising exceptions. *)

let gen_re_opt p_re tag_opt fun_name_opt gen_msg_opt =
  p_re
    (exercise_option
       "" (error_message_opt tag_opt fun_name_opt gen_msg_opt))
;;

let invalid_arg_opt = gen_re_opt Pervasives.invalid_arg
and failwith_opt = gen_re_opt Pervasives.failwith
;;

let internal_error_opt tag_opt fun_name_opt =
  failwith_opt tag_opt fun_name_opt (gen_opt "Internal error, please report.")
;;

let down_re2 f fun_name = downl_option (f None) fun_name
and down_re3 f fun_name msg = downl_option (f None) fun_name (gen_opt msg)
;;

let invalid_arg = down_re3 invalid_arg_opt
and failwith = down_re3 failwith_opt
and internal_error = down_re2 internal_error_opt
;;

let gen_re_qual_opt f tag_opt qual_opt fun_name_opt =
  f tag_opt (qual_opt fun_name_opt)
;;

let invalid_arg_qual_opt = gen_re_qual_opt invalid_arg_opt
and failwith_qual_opt = gen_re_qual_opt failwith_opt
and internal_error_qual_opt = gen_re_qual_opt internal_error_opt
;;

let down_re3_qual f qual fun_name =
  downl_option (f None (lift_option qual)) fun_name
and down_re4_qual f qual fun_name msg =
  downl_option (f None (lift_option qual)) fun_name (gen_opt msg)
;;

let invalid_arg_qual = down_re4_qual invalid_arg_qual_opt
and failwith_qual = down_re4_qual failwith_qual_opt
and internal_error_qual = down_re3_qual internal_error_qual_opt
;;

let p_qual = qualify "Upervasives";;
let p_invalid_arg = invalid_arg_qual p_qual;;

(* Integer arithmetic. *)

type length = Nat.t;;

let length_of_int = Nat.of_int;;
external unsafe_length_of_int : int -> length = "%identity";;
external length_to_int : length -> int = "%identity";;

type index = Nat.t;;

let index_of_int = Nat.of_int;;
external unsafe_index_of_int : int -> index = "%identity";;
external index_to_int : index -> int = "%identity";;

type offset = Nat.t;;

let offset_of_int = Nat.of_int;;
external unsafe_offset_of_int : int -> offset = "%identity";;
external offset_to_int : offset -> int = "%identity";;

(* Floating-point arithmetic. *)

let round_pos x = int_of_float (x +. 0.5);;
let round_neg x = int_of_float (x -. 0.5);;
let round x = if x >= 0. then round_pos x else round_neg x;;

(* Conversion. *)

let char_of_int64 i = char_of_int (Int64.to_int (Int64.logand i 0xffL));;

(* Borrowed from Ustring, which is not compiled yet. *)
let string_init n f =
  if n = 0 then "" else
  let buf = Buffer.create n in
  for i = 0 to pred n do
    Buffer.add_char buf (f i);
  done;
  Buffer.contents buf
;;

let bit_string_of_float f =
  let i = Int64.bits_of_float f in
  string_init 8 (fun j -> char_of_int64 (Int64.shift_right i (8*j)))
;;

let int64_of_char c = Int64.of_int (int_of_char c);;

let float_of_bit_string s =
  if String.length s <> 8 then
    p_invalid_arg "float_of_bit_string" "wrong length" else
  (* Now, we have length s = 8.
     Therefore, the following String.unsafe_get will be safe. *)
  let s7 = String.unsafe_get s 7 in
  let rec loop accu j =
    (* j is decreasing from 6. *)
    if j < 0 then accu else
    (* Now, we have 0 <= j <= 6 < 8 = length s.
       Therefore, the following String.unsafe_get will be safe. *)
    let sj = String.unsafe_get s j in
    let accu = Int64.add (Int64.shift_left accu 8) (int64_of_char sj) in
    loop accu (pred j) in
  Int64.float_of_bits (loop (int64_of_char s7) 6)
;;

(* Data selection. *)

type select =
  | Keep
  | Forget
;;

(* Input/output. *)

let in_channel_length ic = length_of_int (Pervasives.in_channel_length ic);;

let input_string_fmt len =
  Scanf.format_from_string
    (Printf.sprintf "%%%d%,[\000-\255]" (length_to_int len))
    "%s"
;;

let input_string len =
  let fmt = input_string_fmt len in
  fun ic ->
    let ib = Scanf.Scanning.from_channel ic in
    if Scanf.Scanning.end_of_input ib then raise End_of_file else
    let s = Scanf.bscanf ib fmt (fun x -> x) in
    Scanf.Scanning.close_in ib;
    s
;;

let default_input_size = Nat.of_int 2048;;
let default_input_fmt = input_string_fmt default_input_size;;

let iter_channel f ic =
  let ib = Scanf.Scanning.from_channel ic in
  let rec loop () =
    if not (Scanf.Scanning.end_of_input ib) then begin
      Scanf.bscanf ib default_input_fmt f;
      loop ();
    end in
  loop ();
  Scanf.Scanning.close_in ib;
;;

let buffer_add_channel b ic = iter_channel (Buffer.add_string b) ic;;

let buffer_of_ic ic =
  let len = max default_input_size (in_channel_length ic) in
  if len > Nat.of_int Sys.max_string_length then
    failwith "buffer_of_ic" "too long input channel" else
  let b = Buffer.create (length_to_int len) in
  buffer_add_channel b ic;
  b
;;

let string_of_ic ic = Buffer.contents (buffer_of_ic ic);;

let pr_unit pr_string () = pr_string "()";;

let print_unit = pr_unit print_string;;

let prerr_unit = pr_unit prerr_string;;

let pr_bool pr_string b =
  if b then pr_string "true" else pr_string "false"
;;

let print_bool = pr_bool print_string;;

let prerr_bool = pr_bool prerr_string;;

let open_append fname =
  let mode = [Open_wronly; Open_append; Open_creat; Open_text;] in
  let perm = 0o644 in
  open_out_gen mode perm fname
;;

let open_append_bin fname =
  let mode = [Open_wronly; Open_append; Open_creat; Open_binary;] in
  let perm = 0o644 in
  open_out_gen mode perm fname
;;
