(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uuniverse_types.ml,v 1.4 2017/05/31 15:24:06 fclement Exp $*)

module Htbl = Hashtbl;;

module Nat = Unatural;;
module A = Uarray;;
module Idx = Uindex;;

let _0_ = Nat._0_
and _1_ = Nat._1_
and _2_ = Nat._2_
;;

let qual = Upervasives.qualify "Uuniverse";;
let invalid_arg = Upervasives.invalid_arg_qual qual;;

type 'a index = 'a Idx.t;;

type 'a indices = ('a, 'a index) Htbl.t;;
type 'a elements = 'a array;;

type 'a t =
  {
   indices : 'a indices;
   mutable elements : 'a elements;
   mutable cardinal : Nat.t;
  }
and 'a universe = 'a t
;;

(* Expansion phase, or transient state. *)

(* unsafe_make must not create an empty universe, hence n must be
   positive. *)
let unsafe_make n x =
  let idxs = Htbl.create (Nat.to_int n) in
  Htbl.add idxs x (Idx.of_nat _0_);
  let elts = A.make n x in
  { indices = idxs; elements = elts; cardinal = _1_; }
;;

let make n x =
  if n = _0_ then invalid_arg "make" "empty universe" else
  (* Now, we have n > 0.
     Therefore, unsafe_make will be safe. *)
  unsafe_make n x
;;

let copy u =
  { u with
    indices = Htbl.copy u.indices;
    elements = A.copy u.elements; }
;;

let mem u x = Htbl.mem u.indices x;;

(* unsafe_add must not create doubles. *)
let unsafe_add ({ cardinal = n; _; } as u) x =
  let len = A.length u.elements in
  if not (mem u x) then begin
    if len = n then begin
      let elts = A.make (Nat.succ (Nat.mul _2_ len)) x in
      A.blit u.elements _0_ elts _0_ n;
      u.elements <- elts
    end;
    Htbl.add u.indices x (Idx.of_nat n);
    A.set u.elements n x;
    u.cardinal <- Nat.succ n
  end
;;

let index u x = Htbl.find u.indices x;;
let stats u = Htbl.stats u.indices;;
