(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uunix.mli,v 1.32 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 More Unix operations} *)

type program_name = Ufilename.filename
(** The name of a file that contains an executable program. *)
;;

type program_arg = string
(** A program argument is just a string: it should be checked and parsed by
  the program to which the argument is applied. *)
;;

type program_args = program_arg list
(** The list of arguments of a program. The first item of this list should
  be the name of the file that contains the program. *)
;;

type program_argv = program_arg array
(** The array of arguments of a program. The first item of this array should
  be the name of the file that contains the program. *)
;;

type binding = string
(** A string with the format ["<variable>=<value>"] intended to bind a
 variable to a value. For instance, ``x=1'' is intended to bind the variable
 [x] to [1]. *)
;;

type program_environment = binding array
(** The environment of a program execution:
  an array of bindings with the format ["<variable>=<value>"]. *)
;;

val execv : program_name -> program_argv -> 'a;;
val execve : program_name -> program_argv -> program_environment -> 'a;;
val execvp : program_name -> program_argv -> 'a;;
val execvpe : program_name -> program_argv -> program_environment -> 'a
(** Same as usual [Unix.execv*] functions with the additional check that the
  the program filename is indeed the first item of the array of arguments. *)
;;

type process_io_channels = Scanf.Scanning.in_channel * Pervasives.out_channel
(** The input and output channels to interact with a running program. *)
;;

val spawn :
  program_name -> process_io_channels option ref -> process_io_channels
(** [spawn prog io_ref] launches the program in file [prog] if this program has
  not already been launched, and returns a pair of input and output channels to
  interact with the running program [prog].

  The program in file [prog] is launched once and for all at first call to
  [spawn prog io_ref]; any further call to [spawn] with identical arguments has
  no effect: no other instance of [prog] is ever launched.

  In effect, [prog] is launched when [io_ref] is unset (meaning that the
  contents of this reference is [None]); when [prog] has been launched,
  [spawn] sets the contents of [io_ref] to the corresponding channels.
  Conversely, nothing happens when [io_ref] is set (meaning that its contents
  is not [None]).

  More precisely, if the contents of reference [io_ref] is [None], then:
  - [spawn] launches the program in file [prog],
  - [spawn] creates a [Scanf.Scanning] input channel [ib] reading from program
    [prog] standard output (to read data from program [prog]),
  - [spawn] creates a [Pervasives] output channel [oc] writing to program
    [prog] standard input (to write data to program [prog]),
  - [spaw] assigns option [Some (ib, oc)] to reference [io_ref].

  The result of a call to [spawn] is the pair of channels [(ib, oc)] that can
  be used to interact with the running program [prog].
*)
;;

val spawn_init :
  (process_io_channels -> unit) ->
  program_name -> process_io_channels option ref -> process_io_channels
(** [spawn_init init_fun prog io_ref] is similar to {!spawn} [prog io_ref]
  except that the initialization function [init_fun] is called once at the
  effective launch of the program in file [prog].

  More precisely, if the contents of reference [io_ref] is [None], then:
  - [spawn_init] launches the program in file [prog],
  - [spawn_init] creates a [Scanf.Scanning] input channel [ib] reading from
    program [prog] standard output (to read data from program [prog]),
  - [spawn_init] creates a [Pervasives] output channel [oc] writing to
    program [prog] standard input (to write data to program [prog]),
  - [spaw_init] assigns option [Some (ib, oc)] to reference [io_ref],
  - [spaw_init] executes the initialization function [init_fun], evaluating
    the expression [init_fun (ib, oc)].

  The result of a call to [spawn_init] is the pair of channel [(ib, oc)] that
  can be used to interact with the running program [prog].
*)
;;

type process_io_buffers = {
  input_buffer : Scanf.Scanning.scanbuf;
  output_buffer : Buffer.t;
  flush_output : unit -> unit;
  close_io_channels : unit -> unit;
}
(** Machinery based on buffers to interact with a running program.
  The [flush_output] function is intended to effectively send outgoing messages
  by flushing the corresponding output buffer.
  The [close_io_channels] function is intended to quietly close input and
  output channels created at the launch of the running program. *)
;;

val pio_spawn :
  Udebug.specialized_printing_facilities ->
  program_name -> process_io_buffers option ref -> process_io_buffers
(** [spawn pf prog io_ref] is similar to {!spawn} [prog io_ref] except that the
  machinery used to interact with the running program [prog] is based on
  buffers. The additional argument [pf] is used to report a tagged message
  once, when the program [prog] is effectively launched. *)
;;

val std_process_io_buffers : process_io_buffers;;

type pid = int
(** The type of process ID of programs (in short pid). *)
;;

val launch_connected_processes :
  program_name -> program_args -> program_name -> program_args -> pid * pid
(** [launch_connected_processes prog1 args1 prog2 args2] launches
  programs [prog1 args1] and [prog2 args2] with each process reading and
  writing to the corresponding [stdout]/[stdin] of the other process.
  The result of the call is the pair of the process ID of the programs. *)
;;

val launch_connected_processes_with_err_on_channels :
  program_name -> program_args -> out_channel ->
  program_name -> program_args -> out_channel -> pid * pid
(** Same as above with the additional redirection of the standard error of
  each process to the corresponding [out_channel]. *)
;;

val wait_for_normal_termination : pid -> unit
(** Wait for normal termination of the process having the given process ID.
  In case of abnormal termination of process [pid], a relevant message is
  written on [stderr] and the function exits with nonzero error code. *)
;;

val check_termination : pid -> unit
(** Silently wait for the termination of the process having the given process
  ID. *)
;;

val kill_and_wait : pid -> unit
(** Send termination signal to the process having the given process ID, and
  silently wait for its effective termination. *)
;;

val is_dead : pid -> bool
(** Check if the process having the given process ID is dead. *)
;;

val is_one_dead : pid list -> bool
(** Check if one process having a process ID in the given list is dead. *)
;;
